from llm_bridge.logic.chat_generate.model_message_converter.claude_message_converter import convert_message_to_claude
from llm_bridge.logic.chat_generate.model_message_converter.gemini_message_converter import convert_message_to_gemini
from llm_bridge.logic.chat_generate.model_message_converter.openai_message_converter import convert_message_to_openai
from llm_bridge.logic.chat_generate.model_message_converter.openai_responses_message_converter import \
    convert_message_to_openai_responses
from llm_bridge.type.message import Message
from llm_bridge.type.model_message.claude_message import ClaudeMessage
from llm_bridge.type.model_message.gemini_message import GeminiMessage
from llm_bridge.type.model_message.openai_message import OpenAIMessage
from llm_bridge.type.model_message.openai_responses_message import OpenAIResponsesMessage


async def convert_messages_to_openai(messages: list[Message]) -> list[OpenAIMessage]:
    return [await convert_message_to_openai(message) for message in messages]


async def convert_messages_to_openai_responses(messages: list[Message]) -> list[OpenAIResponsesMessage]:
    return [await convert_message_to_openai_responses(message) for message in messages]


async def convert_messages_to_gemini(messages: list[Message]) -> list[GeminiMessage]:
    return [await convert_message_to_gemini(message) for message in messages]


async def convert_messages_to_claude(messages: list[Message]) -> list[ClaudeMessage]:
    return [await convert_message_to_claude(message) for message in messages]



import logging
import re
from typing import AsyncGenerator

import httpx
from fastapi import HTTPException

from llm_bridge.client.implementations.claude.claude_token_counter import count_claude_input_tokens, \
    count_claude_output_tokens
from llm_bridge.client.model_client.claude_client import ClaudeClient
from llm_bridge.type.chat_response import ChatResponse
from llm_bridge.type.serializer import serialize


class StreamClaudeClient(ClaudeClient):
    async def generate_stream_response(self) -> AsyncGenerator[ChatResponse, None]:
        try:
            logging.info(f"messages: {self.messages}")

            input_tokens = await count_claude_input_tokens(
                client=self.client,
                model=self.model,
                system=self.system,
                messages=self.messages
            )

            try:
                async with self.client.messages.stream(
                    model=self.model,
                    max_tokens=4096,
                    temperature=self.temperature,
                    system=self.system,
                    messages=serialize(self.messages)
                ) as stream:
                    async for response_delta in stream.text_stream:
                        chat_response = ChatResponse(text=response_delta)
                        output_tokens = await count_claude_output_tokens(
                            client=self.client,
                            model=self.model,
                            chat_response=chat_response,
                        )
                        yield ChatResponse(
                            text=response_delta,
                            input_tokens=input_tokens,
                            output_tokens=output_tokens,
                        )
            except Exception as e:
                logging.exception(e)
                yield ChatResponse(error=repr(e))

        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            text = e.response.text
            raise HTTPException(status_code=status_code, detail=text)
        except Exception as e:
            logging.exception(e)
            match = re.search(r'\d{3}', str(e))
            if match:
                error_code = int(match.group(0))
            else:
                error_code = 500

            raise HTTPException(status_code=error_code, detail=str(e))



from openai.types.responses import ResponseInputTextParam, ResponseInputImageParam, ResponseOutputTextParam, \
    ResponseInputContentParam, EasyInputMessageParam, ResponseOutputMessageParam

from llm_bridge.logic.chat_generate import media_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type
from llm_bridge.type.message import Message, ContentType
from llm_bridge.type.model_message.openai_responses_message import OpenAIResponsesMessage


async def convert_message_to_openai_responses(message: Message) -> OpenAIResponsesMessage:
    role = message.role.value
    content: list[ResponseInputContentParam | ResponseOutputTextParam] = []

    for content_item in message.contents:
        if content_item.type == ContentType.Text:
            if role == "assistant":
                text_content = ResponseOutputTextParam(type="output_text", text=content_item.data)
            else:
                text_content = ResponseInputTextParam(type="input_text", text=content_item.data)
            content.append(text_content)
        elif content_item.type == ContentType.File:
            file_url = content_item.data
            file_type, sub_type = await get_file_type(file_url)
            if file_type == "image":
                image_url = await media_processor.get_openai_image_content_from_url(file_url)
                image_content = ResponseInputImageParam(
                    type="input_image",
                    image_url=image_url,
                    detail="auto"
                )
                content.append(image_content)
            # TODO: Responses API is currently unsupported for audio input
            # elif file_type == "audio":
            #     encoded_string = await media_processor.get_gpt_audio_content_from_url(file_url)
            #     audio_content = ChatCompletionContentPartInputAudioParam(
            #         type="input_audio",
            #         input_audio=InputAudio(data=encoded_string, format=sub_type)
            #     )
            #     content.append(audio_content)
            else:
                text_content = ResponseInputTextParam(
                    type="input_text",
                    text=f"\n{file_url}: {file_type}/{sub_type} not supported by the current model.\n"
                )
                content.append(text_content)

    if role in ("user", "system"):
        return EasyInputMessageParam(role=role, content=content)
    else:
        return ResponseOutputMessageParam(role=role, content=content)



import logging
import re

import httpx
import openai
from fastapi import HTTPException
from openai import APIStatusError

from llm_bridge.client.implementations.openai.openai_token_couter import count_openai_input_tokens, \
    count_openai_output_tokens
from llm_bridge.client.model_client.openai_client import OpenAIClient
from llm_bridge.type.chat_response import ChatResponse
from llm_bridge.type.serializer import serialize


class NonStreamOpenAIClient(OpenAIClient):
    async def generate_non_stream_response(self) -> ChatResponse:
        try:
            logging.info(f"messages: {self.messages}")

            input_tokens = count_openai_input_tokens(
                messages=self.messages
            )

            completion = await self.client.chat.completions.create(
                messages=serialize(self.messages),
                model=self.model,
                temperature=self.temperature,
                stream=False
            )

            content = completion.choices[0].message.content
            chat_response = ChatResponse(text=content)
            output_tokens = count_openai_output_tokens(chat_response)
            return ChatResponse(
                text=content,
                input_tokens=input_tokens,
                output_tokens=output_tokens,
            )
        except httpx.HTTPStatusError as e:
            status_code = e.response.status_code
            text = e.response.text
            raise HTTPException(status_code=status_code, detail=text)
        except openai.BadRequestError as e:
            status_code = e.status_code
            text = e.message
            raise HTTPException(status_code=status_code, detail=text)
        except APIStatusError as e:
            status_code = e.status_code
            text = e.message
            raise HTTPException(status_code=status_code, detail=text)
        except Exception as e:
            logging.exception(e)
            match = re.search(r'\d{3}', str(e))
            if match:
                error_code = int(match.group(0))
            else:
                error_code = 500

            raise HTTPException(status_code=error_code, detail=str(e))



from dataclasses import is_dataclass, asdict
from enum import Enum
from typing import Any


def serialize(obj: Any) -> str | dict | list | Any:
    if isinstance(obj, Enum):
        return obj.value
    elif is_dataclass(obj):
        return {key: serialize(value) for key, value in asdict(obj).items()}
    elif isinstance(obj, list):
        return [serialize(item) for item in obj]
    elif isinstance(obj, dict):
        return {key: serialize(value) for key, value in obj.items()}
    else:
        return obj



from google.genai import types


async def count_gemini_tokens(
        response: types.GenerateContentResponse
) -> tuple[int, int]:
    usage_metadata = response.usage_metadata
    input_tokens = usage_metadata.prompt_token_count
    output_tokens = usage_metadata.candidates_token_count
    if output_tokens is None:
        output_tokens = usage_metadata.total_token_count - usage_metadata.prompt_token_count
    return input_tokens, output_tokens

